//
// Copyright 2012-2013, Richard Gorton   All rights reserved.
//
/// \file
/// \brief  Logging facility for path profiler - semi-generic
///         A better approach is probably to use a pre-canned 3rd party log facility, but
///         the practice writing this is useful.

#ifndef _LOGGER_H_
#define _LOGGER_H_ 1

#include <stdio.h>
#include <unistd.h>
#include <iostream>
#include <fstream>
#include <ostream>
#include <string>

#include <boost/noncopyable.hpp>

// We don't really want to have the logger be copied, or we could end up with
// truly garbled output in it
class ProfileLogger : boost::noncopyable {
    public:
        ProfileLogger() :
            m_logFileName("ERROR"),
            m_file(NULL)
        { }

        ~ProfileLogger()
        {
            if (NULL != m_file)
            {
                fflush(m_file);
                fclose(m_file);
                m_file = NULL;
            }
        }

        void SetLogFileName(const std::string& file)    { m_logFileName = file; }

        /// \param[in]  message to write to the logfile
        /// \returns    false if there was some sort of error writing the logfile
        bool LogIt(const std::string& msg)              { return LogIt(msg.c_str()); }

        /// \param[in]  message to write to the logfile
        /// \returns    false if there was some sort of error writing the logfile
        bool LogIt(const char* msg)
        {
            static bool InitDone = false;

            if (!InitDone)
            {
                unlink(m_logFileName.c_str());
                m_file = fopen(m_logFileName.c_str(), "w");
                if (NULL == m_file)
                {
                    std::cerr << "Unable to open logfile: ["
                              << m_logFileName
                              << "]\n";
                    return false;
                }
                InitDone = true;
            }

            fprintf(m_file, "%s", msg);
            fflush(m_file);
            return true;
        }

    private:
        std::string     m_logFileName;
        FILE*           m_file;
};

// More than one logger can exist, so long as they are for different logfiles
extern ProfileLogger*    g_Logger;

#endif  // _LOGGER_H_
